package LATMOS::Accounts::Bases::Sql;

use 5.010000;
use strict;
use warnings;

use base qw(LATMOS::Accounts::Bases);
use DBI;

our $VERSION = (q$Rev$ =~ /^Rev: (\d+) /)[0];

=head1 NAME

LATMOS::Ad - Perl extension for blah blah blah

=head1 SYNOPSIS

  use LATMOS::Accounts::Bases;
  my $base = LATMOS::Accounts::Bases->new('unix');
  ...

=head1 DESCRIPTION

Account base access over standard unix file format.

=head1 FUNCTIONS

=cut

=head2 new(%options)

Create a new LATMOS::Ad object for windows AD $domain.

domain / server: either the Ad domain or directly the server

ldap_args is an optionnal list of arguments to pass to L<Net::LDAP>.

=cut

sub new {
    my ($class, %options) = @_;
    
    my $base = {
        db_conn => $options{db_conn},
    };

    bless($base, $class);
}

sub db {
    my ($self) = @_;

    if ($self->{_db} && $self->{_db}->ping) {
        return $self->{_db};
    } else {
        $self->{_db} = DBI->connect_cached(
            'dbi:Pg:' . $self->{db_conn},
            undef, undef,
            {
                RaiseError => 0,
                AutoCommit => 0,
                PrintWarn => 0,
                PrintError => 1,
            }
        ) or return;
        $self->{_db}->do(q{set DATESTYLE to 'DMY'});
        return $self->{_db};
    }
}

sub _canonicals_fields {
    my ($self, $type) = @_;
    $type = lc($type);
    {
        user => {
            uidNumber       => 'uid',
            gidNumber       => 'gid',
            # gecos           => 'gecos',
            homeDirectory   => 'home',
            loginShell      => 'shell',
            #userPassword    => ($self->{use_shadow} ? 'spassword' : 'password'),
        },
        group => {
            gidNumber       => 'gid',
            memberUID       => 'user_list',
        },
    }->{$type}
}

sub list_canonicals_fields {
    my ($self, $type) = @_;
    keys %{ $self->_canonicals_fields($type) || {} }
}

sub get_field_name {
    my ($self, $type, $cfield) = @_;
    my $map = $self->_canonicals_fields($type) || {};
    $map->{$cfield}
}

=head2 load

Read file and load data into memory

=cut

sub load {
    my ($self) = @_;
    return $self->db ? 1 : 0;    
}

sub commit {
    my ($self) = @_;
    $self->db->commit;
}

sub rollback {
    my ($self) = @_;
    $self->db->rollback;
}

sub create_object {
    my ($self, $otype, $id, %data) = @_;

    my $internal_obj = {
        user => 'users',
        group => 'groups',
    }->{$otype};

    if ($self->{$internal_obj}{$id}) { return };
    $self->{$internal_obj}{$id} = { account => $id };
    my $obj = $self->get_object($otype, $id) or return;
    $obj->set_fields(%data);
    $obj
}

1;

__END__

=head1 SEE ALSO

=head1 AUTHOR

Olivier Thauvin, E<lt>olivier.thauvin@latmos.ipsl.frE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2008, 2009 CNRS SA/CETP/LATMOS

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.0 or,
at your option, any later version of Perl 5 you may have available.


=cut
